#! /usr/bin/env python

#
# Copyright (C) 2005  Lorenzo Pallara, lpallara@cineca.it
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os

from dvbobjects.PSI.PAT import *
from dvbobjects.PSI.NIT import *
from dvbobjects.PSI.SDT import *
from dvbobjects.PSI.TDT import *
from dvbobjects.PSI.EIT import *
from dvbobjects.PSI.PMT import *
from dvbobjects.MHP.AIT import *
from dvbobjects.MHP.Descriptors import *
from dvbobjects.DVB.Descriptors import *
from dvbobjects.MPEG.Descriptors import *

#
# NIT
#

nit = network_information_section(
	network_id = 1,
        network_descriptor_loop = [
	    network_descriptor(network_name = "Cineca",), 
            ],
	transport_stream_loop = [
	    transport_stream_loop_item(
		transport_stream_id = 1,
		original_network_id = 1,
		transport_descriptor_loop = [
		    transport_stream_terrestrial_descriptor(
			center_frequency = 0x2367B880, # 594000000 Hz # <<<<<<<<<<<
    			bandwidth = 0, # 8 MHz # <<<<<<<<<<<<
			constellation = 1, # 16-QAM # <<<<<<<<<<<
			hierarchy_information = 0, # non-hierarchical # <<<<<<<<<<<<
			code_rate_HP_stream = 1, # 2/3 # <<<<<<<<<<<
    			code_rate_LP_stream = 1, # 2/3 # <<<<<<<<<<<
    			guard_interval = 0, # 1/32 # <<<<<<<<<<<
    			transmission_mode = 1, # 8k mode # <<<<<<<<<<<
    			other_frequency_flag = 0,
		    ),
		    service_list_descriptor(
			dvb_service_descriptor_loop = [
			    service_descriptor_loop_item(
				service_ID = 1, # <<<<<<<<<<<<
				service_type = 1,
			    ),
			],
		    ),
		    logical_channel_descriptor (
			lcn_service_descriptor_loop = [
			    lcn_service_descriptor_loop_item(
				service_ID = 1,
				logical_channel_number = 1,
				visible_service_flag = 1,
			    ),
			],
		    ),
		],		
	     ),
	  ],
        )
nit.set(
        version_number = 1, # <<<<<<<<<<<<
        section_number = 0,
        last_section_number = 0,
        )


#
# PAT
#

pat = program_association_section(
	transport_stream_id = nit.transport_stream_loop[0].transport_stream_id,
        program_loop = [
    	    program_loop_item(
	        program_number = nit.transport_stream_loop[0].transport_descriptor_loop[1].dvb_service_descriptor_loop[0].service_ID,
    		PID = 1031,
    	    ),  
        ],
        )
pat.set(
        version_number = 1,
        section_number = 0,
        last_section_number = 0,
        )


#
# SDT
#

sdt = service_description_section(
	transport_stream_id = nit.transport_stream_loop[0].transport_stream_id,
	original_network_id = nit.transport_stream_loop[0].original_network_id,
        service_loop = [
	    service_loop_item(
		service_ID = nit.transport_stream_loop[0].transport_descriptor_loop[1].dvb_service_descriptor_loop[0].service_ID,
		EIT_schedule_flag = 1,
		EIT_present_following_flag = 1,
		running_status = 4, # running
		free_CA_mode = 0, # unscrambled
		service_descriptor_loop = [
		    service_descriptor(
			service_type = 1, # digital television service
			service_provider_name = "CINECA",
			service_name = "TEST CHANNEL",
		    ),    
		],
	    ),	
            ],
        )
sdt.set(
        version_number = 1, # <<<<<<<<<<<<
        section_number = 0,
        last_section_number = 0,
        )

#
# TDT, it should be replaced at run time with tstdt
#

tdt = time_date_section(
	year = 105,
	month = 8,
	day = 16,
	hour = 0x11,
	minute = 0x30,
	second = 0x21,
        )
tdt.set(
        version_number = 1,
        section_number = 0,
        last_section_number = 0,
        )


#
# EIT
#

eit = event_information_section(
	table_id = EIT_ACTUAL_TS_PRESENT_FOLLOWING,
	service_id = nit.transport_stream_loop[0].transport_descriptor_loop[1].dvb_service_descriptor_loop[0].service_ID,
	transport_stream_id = nit.transport_stream_loop[0].transport_stream_id, 
	original_network_id = nit.transport_stream_loop[0].original_network_id,
        event_loop = [
	    event_loop_item(
		event_id = 1, # <<<<<<<<<<<<
		start_year = 107, # <<<<<<<<<<<< from 1900
		start_month = 2, # <<<<<<<<<<<<
		start_day = 15, # <<<<<<<<<<<<
		start_hours = 0x00, # <<<<<<<<<<<<
		start_minutes = 0x00, # <<<<<<<<<<<<
		start_seconds = 0x00, # <<<<<<<<<<<<
		duration_hours = 0x23, #  <<<<<<<<<<<<
		duration_minutes = 0x00, # <<<<<<<<<<<<
		duration_seconds = 0x00, # <<<<<<<<<<<<
		running_status = 4, # running
		free_CA_mode = 0, # not scrambled
		event_descriptor_loop = [
		    short_event_descriptor (
			ISO639_language_code = "ita", 
			event_name = "video1",
			text = "example video 1",
		    )    
		],
	    ),	    	
            ],
        )
eit.set(
        version_number = 1, # <<<<<<<<<<<<
        section_number = 0, # <<<<<<<<<<<<
        last_section_number = 1, # <<<<<<<<<<<<
        )


eit_follow = event_information_section(
	table_id = EIT_ACTUAL_TS_PRESENT_FOLLOWING,
	service_id = nit.transport_stream_loop[0].transport_descriptor_loop[1].dvb_service_descriptor_loop[0].service_ID,
	transport_stream_id = nit.transport_stream_loop[0].transport_stream_id, 
	original_network_id = nit.transport_stream_loop[0].original_network_id,
        event_loop = [
	    event_loop_item(
		event_id = 2, # <<<<<<<<<<<<
		start_year = 107, # <<<<<<<<<<<< from 1900
		start_month = 12, # <<<<<<<<<<<<
		start_day = 15, # <<<<<<<<<<<<
		start_hours = 0x23, # <<<<<<<<<<<<
		start_minutes = 0x30, # <<<<<<<<<<<<
		start_seconds = 0x00, # <<<<<<<<<<<<
		duration_hours = 0x12, #  <<<<<<<<<<<<
		duration_minutes = 0x00, # <<<<<<<<<<<<
		duration_seconds = 0x00, # <<<<<<<<<<<<
		running_status = 1, # not running
		free_CA_mode = 0, # not scrambled
		event_descriptor_loop = [
		    short_event_descriptor (
			ISO639_language_code = "ita", 
			event_name = "video2", # <<<<<<<<<<<<
			text = "example video 2", # <<<<<<<<<<<<
		    )    
		],
	    ),
            ],
        )
eit_follow.set(
        version_number = 1, # <<<<<<<<<<<<
        section_number = 1, # <<<<<<<<<<<<
        last_section_number = 1, # <<<<<<<<<<<<
        )




#
# PMT
#	
	
#define program items
dataB_stream = stream_loop_item(
	stream_type = 11,
	elementary_PID = 2002, # <<<<<<<<<<<<
	element_info_descriptor_loop = [
	    association_tag_descriptor(
		association_tag = 0xB, # <<<<<<<<<<<<
		use = 0,
		selector_lenght = 0,
		transaction_id = 0x80000000,
		timeout = 0xFFFFFFFF,
		private_data = "",
	    ),
	    stream_identifier_descriptor(
		component_tag = 0xB, # <<<<<<<<<<<<
	    ),
	    carousel_identifier_descriptor(
		carousel_ID = 1, # <<<<<<<<<<<<
		format_ID = 0, # no enhanced boot
		private_data = "",
	    ),
	    data_broadcast_id_descriptor(
		data_broadcast_ID = 240, # MHP Object Carousel
		ID_selector_bytes = "",
	    ),
	]
	)

dataC_stream = stream_loop_item(
	stream_type = 11,
	elementary_PID = 2003, # <<<<<<<<<<<< 
	element_info_descriptor_loop = [
	    association_tag_descriptor(
		association_tag = 0xC, # <<<<<<<<<<<< 
		use = 0,
		selector_lenght = 0,
		transaction_id = 0x80000000,
		timeout = 0xFFFFFFFF,
		private_data = "",
	    ),
	    stream_identifier_descriptor(
		component_tag = 0xC, # <<<<<<<<<<<< 
	    ),
	    carousel_identifier_descriptor(
		carousel_ID = 2, # <<<<<<<<<<<< 
		format_ID = 0, # no enhanced boot
		private_data = "",
	    ),
	    data_broadcast_id_descriptor(
		data_broadcast_ID = 240, # MHP Object Carousel
		ID_selector_bytes = "",
	    ),
	]
	)

dataD_stream = stream_loop_item(
	stream_type = 11,
	elementary_PID = 2004, # <<<<<<<<<<<< 
	element_info_descriptor_loop = [
	    association_tag_descriptor(
		association_tag = 0xD, # <<<<<<<<<<<< 
		use = 0,
		selector_lenght = 0,
		transaction_id = 0x80000000,
		timeout = 0xFFFFFFFF,
		private_data = "",
	    ),
	    stream_identifier_descriptor(
		component_tag = 0xD, # <<<<<<<<<<<< 
	    ),
	    carousel_identifier_descriptor(
		carousel_ID = 3, # <<<<<<<<<<<< 
		format_ID = 0, # no enhanced boot
		private_data = "",
	    ),
	    data_broadcast_id_descriptor(
		data_broadcast_ID = 240, # MHP Object Carousel
		ID_selector_bytes = "",
	    ),
	]
	)

ste_stream = stream_loop_item(
	stream_type = 12,
	elementary_PID = 2005, # <<<<<<<<<<<< 
	element_info_descriptor_loop = [
	    stream_identifier_descriptor(
		component_tag = 0xE, # <<<<<<<<<<<< 
	    ),
	]
	)


ait_stream = stream_loop_item(
	stream_type = 5,
	elementary_PID = 2001, # <<<<<<<<<<<< 
	element_info_descriptor_loop = [ 
	    application_signalling_descriptor(
		application_type = 1,
		AIT_version = 1,
	    ),
	]	
	)

video_stream = stream_loop_item(
	stream_type = 2,
	elementary_PID = 2064, # <<<<<<<<<<<< 
	element_info_descriptor_loop = []
	)

audio_stream = stream_loop_item(
	stream_type = 1,
	elementary_PID = 2068, # <<<<<<<<<<<< 
	element_info_descriptor_loop = []
	)

# list programs
pmt = program_map_section(
	program_number = nit.transport_stream_loop[0].transport_descriptor_loop[1].dvb_service_descriptor_loop[0].service_ID,
	PCR_PID = 2064, # <<<<<<<<<<<<
	program_info_descriptor_loop = [],	
	stream_loop = [ # <<<<<<<<<<<<
	    video_stream,
	    audio_stream,
	    ait_stream,	    
	    dataB_stream,
	    dataC_stream,
	    dataD_stream,
	    ste_stream,
	    ]
        )    
pmt.set(
        version_number = 1, # <<<<<<<<<<<<
        section_number = 0,
        last_section_number = 0,
        )

#
# AIT
#

# define common descriptors
transport_protocolB_ds = transport_protocol_descriptor(
        protocol_id = MHP_OC_protocol_id,
        transport_protocol_label = 0x01, # <<<<<<<<<<<
        remote_connection = 0,
        component_tag = 0xB, # <<<<<<<<<<<
        )

transport_protocolC_ds = transport_protocol_descriptor(
        protocol_id = MHP_OC_protocol_id,
        transport_protocol_label = 0x02, # <<<<<<<<<<<
        remote_connection = 0,
        component_tag = 0xC, # <<<<<<<<<<<
        )

transport_protocolD_ds = transport_protocol_descriptor(
        protocol_id = MHP_OC_protocol_id,
        transport_protocol_label = 0x03, # <<<<<<<<<<<
        remote_connection = 0,
        component_tag = 0xD, # <<<<<<<<<<<
        )
	
app_ds_b = application_descriptor(
        application_profile = 0x0001,
        version_major = 1,
        version_minor = 0,
        version_micro = 2,
        service_bound_flag = 1,
        visibility = 3, # <<<<<<<<<<<
        application_priority = 1,
        transport_protocol_labels = [transport_protocolB_ds.transport_protocol_label], # <<<<<<<<<<<
        )

app_ds_d = application_descriptor(
        application_profile = 0x0001,
        version_major = 1,
        version_minor = 0,
        version_micro = 2,
        service_bound_flag = 1,
        visibility = 3, # <<<<<<<<<<<
        application_priority = 1,
        transport_protocol_labels = [transport_protocolD_ds.transport_protocol_label], # <<<<<<<<<<<
        )

app_ds_c = application_descriptor(
        application_profile = 0x0001,
        version_major = 1,
        version_minor = 0,
        version_micro = 2,
        service_bound_flag = 1,
        visibility = 3, # <<<<<<<<<<<
        application_priority = 1,
        transport_protocol_labels = [transport_protocolC_ds.transport_protocol_label], # <<<<<<<<<<<
        )

# define main carousel applications
application1 = application_loop_item( # <<<<<<<<<<<<
        organisation_id = 10, 
        application_id = 1000, 
        application_control_code = 2, # PRESENT, 1 AUTOSTART
        application_descriptors_loop = [
            transport_protocolB_ds,
            app_ds_b,
            application_name_descriptor(application_name = "Text input example"),
            dvb_j_application_descriptor(parameters = ["dvb://1/sample8"]),
            dvb_j_application_location_descriptor(
    		base_directory = "/",
		class_path_extension = "",
 		initial_class = "tv.cineca.apps.yambo.Wizard", 
    	    ),
            ]
        )

application2 = application_loop_item( # <<<<<<<<<<<<
        organisation_id = 10, 
        application_id = 1001,
        application_control_code = 2, # PRESENT
        application_descriptors_loop = [
            transport_protocolD_ds,
            app_ds_d,
            application_name_descriptor(application_name = "Test stream event"),
            dvb_j_application_descriptor(parameters = []),
            dvb_j_application_location_descriptor(
    		base_directory = "/",
		class_path_extension = "",
 		initial_class = "Testste", 
    	    ),
            ]
        )

application3 = application_loop_item( # <<<<<<<<<<<<
        organisation_id = 10, 
        application_id = 1002, 
        application_control_code = 2, # PRESENT
        application_descriptors_loop = [
            transport_protocolB_ds,
            app_ds_b,
            application_name_descriptor(application_name = "Class loading example"),
            dvb_j_application_descriptor(parameters = ["dvb://1/sample9"]),
            dvb_j_application_location_descriptor(
    		base_directory = "/",
		class_path_extension = "",
 		initial_class = "tv.cineca.apps.yambo.Wizard", 
    	    ),
            ]
        )
	
#define ait section
ait = application_information_section(
        application_type = DVB_J_application_type,
        common_descriptor_loop = [],
        application_loop = [ # <<<<<<<<<<<<
	    application1,
	    application2,
	    application3,
   	]
        )
ait.set(
        version_number = 1, # <<<<<<<<<<<<
        section_number = 0,
        last_section_number = 0,
	)


#
# PSI marshalling and encapsulation
#

out = open("./nit.sec", "wb")
out.write(nit.pack())
out.close
out = open("./nit.sec", "wb") # python  flush bug
out.close
os.system('/usr/local/bin/sec2ts 16 < ./nit.sec > ./nit.ts')

out = open("./pat.sec", "wb")
out.write(pat.pack())
out.close
out = open("./pat.sec", "wb") # python   flush bug
out.close
os.system('/usr/local/bin/sec2ts 0 < ./pat.sec > ./pat.ts')

out = open("./sdt.sec", "wb")
out.write(sdt.pack())
out.close
out = open("./sdt.sec", "wb") # python   flush bug
out.close
os.system('/usr/local/bin/sec2ts 17 < ./sdt.sec > ./sdt.ts')

out = open("./tdt.sec", "wb")
out.write(tdt.pack())
out.close
out = open("./tdt.sec", "wb") # python   flush bug
out.close
os.system('/usr/local/bin/sec2ts 20 < ./tdt.sec > ./tdt.ts')

out = open("./eit.sec", "wb")
out.write(eit.pack())
out.close
out = open("./eit.sec", "wb") # python   flush bug
out.close
os.system('/usr/local/bin/sec2ts 18 < ./eit.sec > ./eit.ts')

out = open("./eit_follow.sec", "wb")
out.write(eit_follow.pack())
out.close
out = open("./eit_follow.sec", "wb") # python   flush bug
out.close
os.system('/usr/local/bin/sec2ts 18 < ./eit_follow.sec >> ./eit.ts')

out = open("./pmt.sec", "wb")
out.write(pmt.pack())
out.close
out = open("./pmt.sec", "wb") # python   flush bug
out.close
os.system('/usr/local/bin/sec2ts ' + str(pat.program_loop[0].PID) + ' < ./pmt.sec > ./pmt.ts')

out = open("./ait.sec", "wb")
out.write(ait.pack())
out.close
out = open("./ait.sec", "wb") # python   flush bug
out.close
os.system('/usr/local/bin/sec2ts ' + str(ait_stream.elementary_PID) + ' < ./ait.sec > ./ait.ts')
