#! /usr/bin/env python

#
# Copyright (C) 2004  Lorenzo Pallara, lpallara@cineca.it
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# transport_stream_sat and transport_stream_cable descriptors by Kovacs Balazs

import string
from dvbobjects.utils import *
from dvbobjects.MPEG.Descriptor import Descriptor

######################################################################                                                                                                                 
class short_event_descriptor(Descriptor):                                                                                                                                              

    descriptor_tag = 0x4D
    
    def bytes(self):
	assert len(self.ISO639_language_code) == 3
	fmt = "!%dsB%dsB%ds" % (
	    len(self.ISO639_language_code),
	    len(self.event_name),
	    len(self.text),
	)
	return pack(fmt,
	    self.ISO639_language_code,
	    len(self.event_name),
	    self.event_name,
	    len(self.text),
	    self.text,
	)

######################################################################
class parental_rating_descriptor(Descriptor):                                                                                                                                              

    descriptor_tag = 0x55
    
    def bytes(self):
	assert len(self.country_code) == 3
	fmt = "!%dsB" % (
	    len(self.country_code),
	)
	return pack(fmt,
	    self.country_code,
	    self.rating,
	)


######################################################################
class teletext_descriptor(Descriptor):

    descriptor_tag = 0x56

    def bytes(self):
	assert len(self.ISO639_language_code) == 3
	fmt = "!%dsBB" % len(self.ISO639_language_code)
	return pack(fmt,
	    self.ISO639_language_code,
	    (self.type << 3) | 
	    (self.magazine_number & 0x1F),
	    self.page_number,
		)

######################################################################
class vbi_data_descriptor_loop_item(DVBobject):

    def pack(self):
            fmt = "!B"
	    return pack(fmt,
		0x3F & 
		((self.field_parity & 0x01) >> 5) | 
		(self.line_offset & 0x1F),    
		)

class vbi_data_descriptor(Descriptor):

    descriptor_tag = 0x45

    def bytes(self):
	data_bytes = string.join(
		    map(lambda x: x.pack(),
		    self.vbi_data_descriptor_loop),
		    "")
		    
	fmt = "!BB%ds" % len(data_bytes)
	return pack(fmt,
	    self.data_service_id,
	    len(data_bytes),
	    data_bytes,
		)

######################################################################
class stream_identifier_descriptor(Descriptor):

    descriptor_tag = 0x52

    def bytes(self):
	fmt = "!B"
	return pack(fmt,
		    self.component_tag,
		)

######################################################################
class data_broadcast_id_descriptor(Descriptor):

    descriptor_tag = 0x66

    def bytes(self):
        FMT = "!H%ds" % len(self.ID_selector_bytes)
        return pack(FMT,         
                    self.data_broadcast_ID,
                    self.ID_selector_bytes,
                    )            
                                 
######################################################################
class application_signalling_descriptor(Descriptor):

    descriptor_tag = 0x6F

    def bytes(self):
        FMT = "!HB"
        return pack(FMT, 
		self.application_type,
		0xE0 | (self.AIT_version & 0x1F),
                )
		    
######################################################################
class network_descriptor(Descriptor):

    descriptor_tag = 0x40

    def bytes(self):
        fmt = "!B%ds" % len(self.network_name)
        return pack(fmt,
                    len(self.network_name),
                    self.network_name,
                    )
		    
######################################################################
class service_descriptor(Descriptor):

    descriptor_tag = 0x48

    def bytes(self):
        fmt = "!BB%dsB%ds" % (len(self.service_provider_name), len(self.service_name))
        return pack(fmt,
		    self.service_type,
		    len(self.service_provider_name),
		    self.service_provider_name,
		    len(self.service_name),
		    self.service_name,
                    )


######################################################################
class transport_stream_terrestrial_descriptor(Descriptor):

    descriptor_tag = 0x5a

    def bytes(self):
        fmt = "!LBBBL"
        return pack(fmt,
                    self.center_frequency,
		    (self.bandwidth << 5) | 0x1f,
		    (self.constellation << 6) | (self.hierarchy_information << 3)| (self.code_rate_HP_stream),
		    (self.code_rate_LP_stream << 6) | (self.guard_interval << 3) | (self.transmission_mode << 2) | (self.other_frequency_flag),
		    0xffffffff,
                    )

######################################################################
class transport_stream_sat_descriptor(Descriptor):

    descriptor_tag = 0x43
 
    def bytes(self):
	fmt = "!LHBL"
	return pack(fmt,
		self.frequency,
		self.orbital_position,
		(self.west_east_flag << 7) | (self.polarization << 5) |  self.modulation,
		(self.symbol_rate << 4)| self.FEC_inner,
	)
 
#######################################################################
class transport_stream_cable_descriptor(Descriptor):

    descriptor_tag = 0x44
 
    def bytes(self):
	fmt = "!LHBL"
	return pack(fmt,
	    self.frequency,
	    (self.FEC_outer) | 0xFFF0,
	    (self.modulation),
	    (self.symbol_rate << 4)| (self.FEC_inner),
	)
 		    
######################################################################
class service_descriptor_loop_item(DVBobject):

    def pack(self):
	fmt = "!HB"
	return pack(fmt,
	    self.service_ID,
	    self.service_type,
	)

class service_list_descriptor(Descriptor):

    descriptor_tag = 0x41

    def bytes(self):
        dvb_service_bytes = string.join(
	    map(lambda x: x.pack(),	
                self.dvb_service_descriptor_loop),
            "")								    

        FMT = "!%ds" % len(dvb_service_bytes)
        return pack(FMT,
                    dvb_service_bytes,
                    )

######################################################################
class lcn_service_descriptor_loop_item(DVBobject):

    def pack(self):
	fmt = "!HH"
	return pack(fmt,
	    self.service_ID,
	    ((self.visible_service_flag << 15) | 0x7C00 | self.logical_channel_number),
	)

class logical_channel_descriptor(Descriptor):

    descriptor_tag = 0x83

    def bytes(self):
        lcn_service_bytes = string.join(
	    map(lambda x: x.pack(),	
                self.lcn_service_descriptor_loop),
            "")								    

        FMT = "!%ds" % len(lcn_service_bytes)
        return pack(FMT,
                    lcn_service_bytes,
                    )

######################################################################
class component_descriptor(Descriptor):

    descriptor_tag = 0x50

    def bytes(self):
        fmt = "!BBB%ds%ds" % (
			len(self.ISO_639_language_code),
			len(self.text_char),
			)
        return pack(fmt,
                    0xF0 | (self.stream_content),
                    self.component_type,
		    self.component_tag,
		    self.ISO_639_language_code,
		    self.text_char,
        )

######################################################################
class PDC_descriptor(Descriptor):

    descriptor_tag = 0x69

    def bytes(self):
        fmt = "!BH"
        return pack(fmt,
                    (0xF0) | (self.day >> 1),
		    (self.day << 15) | (self.month << 11) | (self.hour << 6) | (self.minute),
        )
					
