#! /usr/bin/env python

#
# Copyright (C) 2007  Lorenzo Pallara, lpallara@cineca.it
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Justdvb-It object carousel tools encapsulation, you shouldn't need to change this

# Paramas:
# 1 - carousel directory name
# 2 - modules version
# 3 - association aka common tag 
# 4 - pid 
# 5 - carousel id 
# 6 - compressed ( yes|no)

import getopt
import sys
import os
from dvbobjects.utils.mod2sec import *
from dvbobjects.utils.file2mod import *

ocmod = './ocmod/'
ocsec = './ocsec/'

def Usage(return_code = 1):
	print("Usage: %s\n"
		"1 - carousel directory name\n"
		"2 - modules version\n"
		"3 - compressed (yes|no)\n"
		"4 - association/common tag\n" 
		"5 - pid\n" 
		"6 - carousel id" ) % (
		sys.argv[0])
	sys.exit(return_code)

def  DelOneLevel(dir):
	if os.path.exists(dir):
		list = os.listdir(dir)
		for file in list:
			os.remove(dir + '/' + file)
		os.rmdir(dir)

def UpdateModules(diroc, dirmod, compression, pid):

	if not os.path.exists(ocsec):
		os.mkdir(ocsec)	
	dirsec = ocsec + diroc
	DelOneLevel(dirsec)
	os.mkdir(dirsec)

	#compress modules?
	if compression in ['yes']:
		list = os.listdir(dirmod)
		for file in list:
			if os.path.splitext(file)[1] in ['.mod']:
				filepath = dirmod + '/' + file
				out = open(filepath + '.size', 'wb')
				out.write(str(os.stat(filepath)[ST_SIZE]))
				out.flush()
				out.close()
				os.system('/usr/local/bin/zpipe < ' + filepath + ' > ' + filepath  + '.z' )
				os.rename(filepath  + '.z' ,  filepath  )

	#create sections from modules
	BuildCarousel(dirmod, dirsec)

	#enqueue sections in a single file
	if os.path.exists('./' + diroc + '.sec'):
		os.remove('./' + diroc + '.sec')
	out = open('./' + diroc + '.sec', 'wb')
	list = os.listdir(dirsec)
	for file in list:
		cat = open(dirsec + '/' + file, 'rb')
		out.write(cat.read())
		cat.close()
	out.flush()
	out.close()

	#create carousel transport stream
	os.system('/usr/local/bin/sec2ts ' + str(pid) + ' < ./' + diroc +'.sec > ./'  + diroc +'.ts')
	

def UpdateCarousel(diroc, version, compression, tag, pid, cid):

	#set up directories
	if not os.path.exists(ocmod):
		os.mkdir(ocmod)
	dirmod = ocmod + diroc 
	DelOneLevel(dirmod)
	os.mkdir(dirmod)
	
	#create modules
	GenModules(diroc, dirmod, cid, cid, tag, version)
	
	#update modules
	UpdateModules(diroc, dirmod, compression, pid)


if __name__ == '__main__':

	# check params
	try:
		opts, args = getopt.getopt(sys.argv[1:], ' ')
		
	except getopt.error:
		Usage(0)
	if len(args) <> 6:
		Usage(0)
	diroc, version, compression, tag, pid, cid = args
	UpdateCarousel(diroc, int(version,16), compression, int(tag, 16), int(pid), int(cid))
