#! /usr/bin/env python

#
# Copyright (C) 2007  Lorenzo Pallara, lpallara@cineca.it
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os

from dvbobjects.PSI.PAT import *
from dvbobjects.PSI.NIT import *
from dvbobjects.PSI.SDT import *
from dvbobjects.PSI.TDT import *
from dvbobjects.PSI.EIT import *
from dvbobjects.PSI.PMT import *
from dvbobjects.MHP.AIT import *
from dvbobjects.MHP.Descriptors import *
from dvbobjects.DVB.Descriptors import *
from dvbobjects.MPEG.Descriptors import *


# Main readme:
# This python scripts define the components that are shared beteween the different scripts 
# so that all these parameters are defined only once and here
# To add new PID you need to define them here

# # # service defines # # #

#
# service 1 description
#
service1 = service_descriptor_loop_item(
	service_ID = 1,
	service_type = 1, # digital television
)
#
# service 1 description
#

#
# transport stream 1 description
#
transport_stream1 =  transport_stream_loop_item(
		transport_stream_id = 1,
		original_network_id = 1,
		transport_descriptor_loop = [
			transport_stream_terrestrial_descriptor(
				center_frequency = 0x2367B880, # 594000000 Hz 
				bandwidth = 0, # 8 MHz 
				constellation = 1, # 16-QAM 
				hierarchy_information = 0, # non-hierarchical 
				code_rate_HP_stream = 1, # 2/3 
				code_rate_LP_stream = 1, # 2/3 
				guard_interval = 0, # 1/32 
				transmission_mode = 1, # 8k mode 
				other_frequency_flag = 0,
			),
			service_list_descriptor(
				dvb_service_descriptor_loop = [
					service1,
				],
			),
			logical_channel_descriptor (
				lcn_service_descriptor_loop = [
					lcn_service_descriptor_loop_item(
						service_ID = service1.service_ID,
						logical_channel_number = 1,
						visible_service_flag = 1,
					),
				],
			),
		],
	)
#
# transport stream 1 description
#


# # # PID defines # # #

#
# service 1 PID description
#
program1 = program_loop_item(
	program_number = service1.service_ID,
	PID = 1031,
)
#
# service 1 PID description
#

#
# Define video PID and descriptors
#
video_stream = stream_loop_item(
	stream_type = 2,
	elementary_PID = 2064,
	element_info_descriptor_loop = [
		]
	)
#
# Define video PID and descriptors
#

#
# Define audio PID and descriptors
#
audio_stream = stream_loop_item(
	stream_type = 1,
	elementary_PID = 2068,
	element_info_descriptor_loop = [
		]
	)
#
# Define audio PID and descriptors
#

#
# Define DSMCC B PID and descriptors
#
transport_protocolB_ds = transport_protocol_descriptor(
        protocol_id = MHP_OC_protocol_id,
        transport_protocol_label = 0x01,
        remote_connection = 0,
        component_tag = 0xB,
        )
dataB_stream = stream_loop_item(
	stream_type = 11,
	elementary_PID = 2002,
	element_info_descriptor_loop = [
		association_tag_descriptor(
			association_tag = transport_protocolB_ds.component_tag, 
			use = 0,
			selector_lenght = 0,
			transaction_id = 0x80000000,
			timeout = 0xFFFFFFFF,
			private_data = "",
		),
		stream_identifier_descriptor(
			component_tag = transport_protocolB_ds.component_tag,
		),
		carousel_identifier_descriptor(
			carousel_ID = transport_protocolB_ds.transport_protocol_label ,
			format_ID = 0, # no enhanced boot
			private_data = "",
		),
		data_broadcast_id_descriptor(
			data_broadcast_ID = 240, # MHP Object Carousel
			ID_selector_bytes = "",
		),
		]
	)

#
# Define DSMCC B PID and descriptors
#

#
# Define DSMCC C PID and descriptors
#
transport_protocolC_ds = transport_protocol_descriptor(
        protocol_id = MHP_OC_protocol_id,
        transport_protocol_label = 0x02,
        remote_connection = 0,
        component_tag = 0xC, 
        )

dataC_stream = stream_loop_item(
	stream_type = 11,
	elementary_PID = 2003,
	element_info_descriptor_loop = [
		association_tag_descriptor(
			association_tag = transport_protocolC_ds.component_tag, 
			use = 0,
			selector_lenght = 0,
			transaction_id = 0x80000000,
			timeout = 0xFFFFFFFF,
			private_data = "",
		),
		stream_identifier_descriptor(
			component_tag = transport_protocolC_ds.component_tag,
		),
		carousel_identifier_descriptor(
			carousel_ID = transport_protocolC_ds.transport_protocol_label ,
			format_ID = 0, # no enhanced boot
			private_data = "",
		),
		data_broadcast_id_descriptor(
			data_broadcast_ID = 240, # MHP Object Carousel
			ID_selector_bytes = "",
		),
		]
	)
	
#
# Define DSMCC C PID and descriptors
#

#
# Define DSMCC D PID and descriptors
#
transport_protocolD_ds = transport_protocol_descriptor(
        protocol_id = MHP_OC_protocol_id,
        transport_protocol_label = 0x03, 
        remote_connection = 0,
        component_tag = 0xD, 
        )
dataD_stream = stream_loop_item(
	stream_type = 11,
	elementary_PID = 2004,
	element_info_descriptor_loop = [
		association_tag_descriptor(
			association_tag = transport_protocolD_ds.component_tag, 
			use = 0,
			selector_lenght = 0,
			transaction_id = 0x80000000,
			timeout = 0xFFFFFFFF,
			private_data = "",
		),
		stream_identifier_descriptor(
			component_tag = transport_protocolD_ds.component_tag,
		),
		carousel_identifier_descriptor(
			carousel_ID = transport_protocolD_ds.transport_protocol_label ,
			format_ID = 0, # no enhanced boot
			private_data = "",
		),
		data_broadcast_id_descriptor(
			data_broadcast_ID = 240, # MHP Object Carousel
			ID_selector_bytes = "",
		),
		]
	)	
	
#
# Define DSMCC D PID and descriptors
#

#
# Define Stream event PID and descriptors
#
ste1_stream = stream_loop_item(
	stream_type = 12,
	elementary_PID = 2005,  
	element_info_descriptor_loop = [
	    stream_identifier_descriptor(
		component_tag = 0xE, 
	    ),
	]
	)

#
# Define Stream event PID and descriptors
#

#
# Define Ait stream PID and descriptors
#
ait1_stream = stream_loop_item(
	stream_type = 5,
	elementary_PID = 2001, 
	element_info_descriptor_loop = [ 
		application_signalling_descriptor(
			application_type = 1,
			AIT_version = 1,
		),
	]	
	)

#
# Define Ait stream PID and descriptors
#

