#! /usr/bin/env python

#
# Copyright (C) 2007  Lorenzo Pallara, lpallara@cineca.it
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# Main readme:
# This transport generator script defines how the transport streams are multiplexed, 
# every time you need to add a PID stream it is necessary to add it here to make it multiplexed.
# In this example the null packets of a input transport stream with audio, video and pcr
# are replaced by three object carousel muxed between them and many psi tables are added.
# Every time the *update python scripts are called the multiplexing will take in account
# the new generated files

from os import *
import psimain
import justdvbit
from secupdate import *

# generate and add main psi tables to output
import nit
UpdateSection(nit.sec)
justdvbit.AddTableToNullShaper(nit.sec)

import pat
UpdateSection(pat.sec)
justdvbit.AddTableToNullShaper(pat.sec)

import sdt
UpdateSection(sdt.sec)
justdvbit.AddTableToNullShaper(sdt.sec)

import tdt
UpdateSection(tdt.sec)
justdvbit.AddTableToNullShaper(tdt.sec)

# generate epg streams, 1 stream usually carry 2 descriptor with current and next event
import eit
UpdateSection(eit.sec)
justdvbit.AddTableToNullShaper(eit.sec)

# generate pmt stream, 1 stream usually carry 1 decriptor for 1 service
import pmt1
UpdateSection(pmt1.sec)
justdvbit.AddTableToNullShaper(pmt1.sec)

# generate ait1 stream, 1 stream usually carry 1 descriptor with many applications
import ait1
UpdateSection(ait1.sec)
justdvbit.AddTableToNullShaper(ait1.sec)

# generate dsmcc stream event streams
import ste1
UpdateSection(ste1.sec)
justdvbit.AddTableToNullShaper(ste1.sec)

# Repeats in a loop a CBR TS with an audio/video generated by MainConcept's mpeg2 encoder stripped from original PSI packets and filled with null packets 
justdvbit.InputTransport('input.ts')

# generate carousel from 3 directories: oc1, oc2, oc3
# parameter are: input directory,  version, compression on or off, carousel description from psimain
justdvbit.GenerateCarousel('./oc1', 1,  'no', psimain.dataB_stream, psimain.transport_protocolB_ds)
justdvbit.AddCarouselToMux('oc1', '140000')

justdvbit.GenerateCarousel('./oc2', 1,  "no", psimain.dataC_stream, psimain.transport_protocolC_ds)
justdvbit.AddCarouselToMux('oc2', "640000")

justdvbit.GenerateCarousel('./oc3',  1,  'yes', psimain.dataD_stream, psimain.transport_protocolD_ds)
justdvbit.AddCarouselToMux('oc3', '220000')

# Muxes carousels the result is a TS with the bitrate equals to the addition of all the bitrates
justdvbit.MuxCarousels()

# Changes dynamically input file null packets with the PSI tables and the muxed object carousel
justdvbit.ShapeTransport()

# Fix tdt with local time and date
justdvbit.ClockTransport()

# output final tool, you should add the final command line for your hw, check readme.txt for hw
#system('cat ./fifotimed.ts > /dev/null &')
#example:
system('/usr/local/bin/DtPlay ./fifotimed.ts -t 110 -mt OFDM -mC QPSK -mc 3/4 -mG 1/16 -mf 714 &')

