/*  
 * Copyright (C) 2004  Lorenzo Pallara, lpallara@cineca.it 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <netinet/in.h>

#include <fcntl.h>
#include <unistd.h> 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <inttypes.h>
#include <unistd.h>

#define PES_HEADER_SIZE 19
#define ES_HEADER_SIZE 4
#define ES_HEADER_MAX_SIZE ES_HEADER_SIZE * 2 
#define PTS_MAX 8589934592LL

void stamp_ts (unsigned long long int ts, unsigned char* buffer) 
{

	if (buffer) {
		buffer[0] = ((ts >> 29) & 0x0F) | 0x01;
		buffer[1] = (ts >> 22) & 0xFF;
		buffer[2] = ((ts >> 14) & 0xFF ) | 0x01;
		buffer[3] = (ts >> 7) & 0xFF;
		buffer[4] = ((ts << 1) & 0xFF ) | 0x01;
	}											

}

int main(int argc, char *argv[])
{
	int bframe;
	int skip_next_header;
	int byte_read;
	FILE* file_es;
	unsigned long long int dts;
	unsigned long long int pts;
	unsigned long long int ts;
	unsigned char stream_id;
	unsigned char pes_header[PES_HEADER_SIZE];
	unsigned char es_header[ES_HEADER_MAX_SIZE];
	unsigned int pts_step;
	
	/* Open pes file */
	if (argc > 3) {
		file_es = fopen(argv[1], "rb");
		stream_id = atoi(argv[2]);
		pts_step = atoi(argv[3]);
	} else {
		fprintf(stderr, "Usage: 'esvideo2pes video.es stream_id pts_step'\n");
		fprintf(stderr, "Valid bitmaps for videos are 1110xxxx\n");
		fprintf(stderr, "Example for video stream id 0: esvideo2pes video.es 224 3600 \n");
		return 2;
	}
	if (file_es == 0) {
		fprintf(stderr, "Can't find file %s\n", argv[1]);
		return 2;
	}

	/* Init. default pack header */
	pes_header[0] = 0x00;
	pes_header[1] = 0x00;
	pes_header[2] = 0x01;
	pes_header[3] = stream_id;
	pes_header[4] = 0x00;
	pes_header[5] = 0x00;
	pes_header[6] = 0x80; /* no scrambling, no priority, no alignment defined, no copyright, no copy */
	pes_header[7] = 0x80; /* pts, no escr, no es rate, no dsm trick, no extension flag, no additional copy info, no crc flag */
	pes_header[8] = 0x05; /* pts */ 

	/* Start the process */
	bframe = 1;
	skip_next_header = 0;
	ts = 0;
	byte_read = fread(es_header, 1, ES_HEADER_SIZE, file_es);
	while(byte_read) {
		
		if ( !skip_next_header && es_header[0] == 0x00 && es_header[1] == 0x00 && es_header[2] == 0x01 && (es_header[3] == 0x00 ||  es_header[3] == 0xB3 || es_header[3] == 0xB8 )) { 
			
			if (es_header[3] == 0x00 ) {
				byte_read = fread(es_header + ES_HEADER_SIZE, 1, ES_HEADER_SIZE, file_es);
				bframe = ((es_header[5] & 0x38) >> 3);
				bframe = (bframe == 0x03);
			} else { /* next one should be an I frame */
				bframe = 0; 
				byte_read = 0; 
				skip_next_header = 1;
			}
			
			if (!bframe) {
				bframe = 1;
				pes_header[7] = 0xC0; /* add dts flag */
				pes_header[8] = 0x0A; /* pts and dts */			
				
				/* pts is delayed */
				pts = ts + pts_step * 3;
				if (pts > PTS_MAX) {
					pts -= PTS_MAX;
				}
				stamp_ts(pts, pes_header + 9);
				pes_header[9] &= 0x0F;
				pes_header[9] |= 0x30;
			
				/* dts is now */
				dts = ts;
				stamp_ts(dts, pes_header + 14);
				pes_header[14] &= 0x0F;
				pes_header[14] |= 0x10;
				
				/* go on count from now */
				ts += pts_step;
				if (ts > PTS_MAX) {
					ts -= PTS_MAX;
				}
				
				fwrite(pes_header, 1, PES_HEADER_SIZE, stdout);
			} else {
				pes_header[7] = 0x80; /* only pts */
				pes_header[8] = 0x05; /* pts */

				/* pts is now*/
				pts = ts;
				stamp_ts(pts, pes_header + 9);
				pes_header[9] &= 0x0F;
				pes_header[9] |= 0x20;
				
				ts += pts_step;
				if (ts > PTS_MAX) {
					ts -= PTS_MAX;
				}	
				fwrite(pes_header, 1, PES_HEADER_SIZE - 5, stdout);
			}
			if (byte_read) {
			    fwrite(es_header, 1, ES_HEADER_MAX_SIZE, stdout);
			} else {
			    fwrite(es_header, 1, ES_HEADER_SIZE, stdout);			
			}
			byte_read += fread(es_header, 1, ES_HEADER_SIZE, file_es);
		
		} else if ( skip_next_header && es_header[0] == 0x00 && es_header[1] == 0x00 && es_header[2] == 0x01 && es_header[3] == 0x00) { 
			skip_next_header = 0;	
			fwrite(es_header, 1, ES_HEADER_SIZE, stdout);
			byte_read = fread(es_header, 1, ES_HEADER_SIZE, file_es);
		} else {
			fwrite(es_header, 1, 1, stdout);	
			es_header[0] = es_header[1];
			es_header[1] = es_header[2];
			es_header[2] = es_header[3];
			byte_read = fread(es_header + 3, 1, 1, file_es);
		}

	}

	return 0;
}

