/*  
 * Copyright (C) 2005  Lorenzo Pallara, lpallara@cineca.it 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <netinet/in.h>

#include <fcntl.h>
#include <unistd.h> 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <inttypes.h>

#define TS_PACKET_SIZE 188

typedef struct list {
	int			fd;
	double			bps;
	double			output_packets;
	struct list*		next; 
} fd_list;

fd_list* g_fd_list = 0;
fd_list* g_fd_list_last = 0;
double g_output_packets = 0;

int enqueue_fd(int fd, int bps) {
    
	fd_list* fd_new_list;
    
	fd_new_list = (fd_list*) malloc(sizeof(fd_list));
	fd_new_list->fd = fd;
	fd_new_list->bps = bps;
	fd_new_list->output_packets = 0;
	fd_new_list->next = 0;	
	if (g_fd_list == 0) {
		g_fd_list = fd_new_list;
		g_fd_list_last = fd_new_list;
	} else {
		g_fd_list_last->next = fd_new_list;
		g_fd_list_last = fd_new_list;
	}
    
	return (fd < 0);
}

fd_list* choose_fd(void) {

	int not_chosen = 1;
	double min = 0;
	double min_packets = 0;
	double temp = 0;
	fd_list* fd_list_index;
	fd_list* result;
	
	fd_list_index = g_fd_list;
	while(fd_list_index != 0) {
		
		/* search the min */
		temp = (fd_list_index->output_packets * TS_PACKET_SIZE * 8) / fd_list_index->bps;
		if (not_chosen || temp < min) {
			min = temp;
			min_packets = fd_list_index->output_packets;			
			result = fd_list_index;
			not_chosen = 0;
		}
		
		fd_list_index = fd_list_index->next;
	}
	
	result->output_packets++;
	g_output_packets++;
	if (g_output_packets < 0) {
		/* total output counter will go overflow before the others counters */
		fprintf(stderr, "Managed output counter overflow\n");	
		fd_list_index = g_fd_list;
		while(fd_list_index != 0) {
			fd_list_index->output_packets -= min_packets;
			fd_list_index = fd_list_index->next; 
		}
		g_output_packets = 1;
	} 
	return result;
}

void Usage (void) {
	fprintf(stderr, "Usage: 'tscbrmuxer b:bitrate_pat pat.ts b:bitrate_cat cat.ts b:pmt1_bitrate pmt1.ts b:nit_bitrate nit.ts ... where bitrate in 'b:bitrate file.ts' is in bit per second\n");
}

int main(int argc, char *argv[])
{
	int fd;
	int bps;
	double output_bitrate;
	unsigned int i;
	unsigned char ts_packet[TS_PACKET_SIZE];
	fd_list* chosen_fd; 
	
	/* Parse command line */ 
	if (argc < 3) {
		Usage();
		return 2;
	}
	
	i = 1;
	while (i < argc) {
		if (argv[i][0] == 'b' && argv[i][1] == ':') {
			bps = atoi(&(argv[i][2]));
			fd = open(argv[i+1], O_RDONLY);
			if (bps > 0 && fd >= 0) {
				if(enqueue_fd(fd, bps))
					return 2;
			} else if (bps < 0){
				fprintf(stderr, "specified bps is not positive\n");	
				return 2;
			} else if (fd < 0) {
				fprintf(stderr, "can't find file %s\n", argv[i+1]);
				return 2;
			}
			output_bitrate += bps;
			i+=2;
		} else {
			Usage();
			return 2;
		}
	}
	fprintf(stderr, "output bitrate: %f\n", output_bitrate);

	/* mux them */
	while(1) {
	
		chosen_fd = choose_fd();
		read(chosen_fd->fd, ts_packet, TS_PACKET_SIZE);
		write(STDOUT_FILENO, ts_packet, TS_PACKET_SIZE);
	}
	
	return 0;
}
